﻿using System.ComponentModel;
using System.Runtime.CompilerServices;
using System.Windows;
using System.Windows.Input;
using Szoveg.Annotations;

namespace Szoveg
{
	public class ViewModel:IViewModel,INotifyPropertyChanged
	{
		/// <summary>
		/// Modell objektum referenciájának tárolásásra szolgáló adattag. 
		/// A ViewModel és a View között adatkötés áll fenn. Ha az adat módosul, akkor 
		/// értesíteni kell a View-t. Ennek ajánlott módja az INotifyPropertyChanged
		/// interfész megvalósítása, azaz PropertyChanged esemény előidézése.
		/// </summary>
		private IModel model;

		private IView wnd;
		public ViewModel(IView view)
		{
			// A View referenciájának tárolása
			wnd = view;
			// Parancscsatoló objektum létrehozása a mentés funkcióhoz
			var cbMentés =
				new CommandBinding(ApplicationCommands.Save);
			// Eseménykezelő metódus parancshoz rendelése.
			cbMentés.Executed += cbMentés_Executed;

			// Parancscsatoló objektum létrehozása a megnyitás funkcióhoz
			var cbMegnyitás =
				new CommandBinding(ApplicationCommands.Open);
			// Eseménykezelő metódus parancshoz rendelése
			cbMegnyitás.Executed += cbMegnyitás_Executed;

			// Parancscsatoló objektum létrehozása az új dokumentum funkcióhoz
			var cbÚj = new CommandBinding(
					ApplicationCommands.New);
			// Eseménykezelő metódus parancshoz rendelése
			cbÚj.Executed += cbÚj_Executed;

			var cbKilépés = new CommandBinding(
				ApplicationCommands.Close);
			cbKilépés.Executed += cbKilépés_Executed;

			// Parancscsatoló objektumok felvétele az ablak parancscsatoló listájába.
			wnd.CommandBindings.Add(cbMentés);
			wnd.CommandBindings.Add(cbMegnyitás);
			wnd.CommandBindings.Add(cbÚj);
			wnd.CommandBindings.Add(cbKilépés);
			wnd.DataContext = this;

			// Modell objektum létrehozása
			model=new Model(this);
		}
		public string Szöveg
		{
			get { return model.Szöveg; }
			set
			{
				model.Szöveg = value;
				// Előidézi a PropertyChanged eseményt és átadja az érintett 
				// tulajdonság nevét.
				Értesít(nameof(Szöveg));
			}
		}
		public void cbÚj_Executed(object sender, ExecutedRoutedEventArgs e)
		{
			VáltozásEllenőrzés();
			// Szövegmező tartalmának ürítése.
			Szöveg="";
			model.VoltVáltozás = false;
		}
		public void cbMegnyitás_Executed(object sender, ExecutedRoutedEventArgs e)
		{
			// Ha megváltozott a szerkesztőmező értéke a legutóbbi mentés óta.
			VáltozásEllenőrzés();
			model.Betöltés();
			MessageBox.Show("Állomány betöltve!");
		}
		public void cbMentés_Executed(object sender, ExecutedRoutedEventArgs e)
		{
			model.Mentés();
			MessageBox.Show("Állomány mentve!");
		}
		public void cbKilépés_Executed(object sender, ExecutedRoutedEventArgs e)
		{
			VáltozásEllenőrzés();
			// Alkalmazás leállítása
			Application.Current.Shutdown();
		}
		private void VáltozásEllenőrzés()
		{
			// Ha megváltozott a szerkesztőmező értéke a legutóbbi mentés óta
			if (!model.VoltVáltozás) return;
			// Akar-e menteni a felhasználó?
			var res = MessageBox.Show("Akarsz menteni?", "Kérdés",
				MessageBoxButton.YesNo, MessageBoxImage.Question);
			if (res != MessageBoxResult.Yes) return;
			model.Mentés();
			MessageBox.Show("Állomány mentve!");
		}


		#region INotifyPropertyChanged
		public event PropertyChangedEventHandler PropertyChanged;

		[NotifyPropertyChangedInvocator]
		protected virtual void Értesít([CallerMemberName] string propertyName = null)
		{
			// Ha van feliratkozó a PropertyChanged eseményre, akkor előidézi az eseményt.
			PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
		}
		#endregion
	}
}
